@@ -1,14 +1,18 @@
 cesCoefNames <- function( nExog, vrs, returnRho1 = TRUE, returnRho2 = TRUE, 
-      returnRho = TRUE, nested = FALSE ) {
+      returnRho = TRUE, nested = FALSE, withTime = FALSE ) {
 
+   result <- "gamma"
+   if( withTime ) {
+      result <- c( result, "lambda" )
+   }
    if( nExog == 2 ) {
-      result <- c( "gamma", "delta" )
+      result <- c( result, "delta" )
    } else if( !nested ) {
-      result <- c( "gamma", paste( "delta", 1:nExog, sep = "_" ) )
+      result <- c( result, paste( "delta", 1:nExog, sep = "_" ) )
    } else if( nested && nExog == 3 ) {
-      result <- c( "gamma_1", "gamma_2", "delta_1", "delta_2" )
+      result <- c( result, "delta_1", "delta" )
    } else if( nested && nExog == 4 ) {
-      result <- c( "gamma", "delta_1", "delta_2", "delta_3" )
+      result <- c( result, "delta_1", "delta_2", "delta" )
    } else {
       stop( "internal error: non-supported arguments to cesCoefNames()" )
    }
